SystemOrganization addCategory: #PlayerST!
SystemOrganization addCategory: #'PlayerST-Kernel'!
SystemOrganization addCategory: #'PlayerST-Tests'!
SystemOrganization addCategory: #'PlayerST-Interfaces'!

TestCase subclass: #PlayerClientTest
	instanceVariableNames: 'playerClient'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Tests'!

!PlayerClientTest methodsFor: 'running' stamp: 'zz 2/4/2009 11:29'!
setUp
	playerClient := PlayerClient new! !

!PlayerClientTest methodsFor: 'testing' stamp: 'SergeStinckwich 2/12/2009 15:23'!
testDecodingByteArrayAsAPlayerDevAddr
	| b pda |
	b := #(0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 ) asByteArray.
	pda := playerClient decodeDevAddressFrom: b.
	self
		assert: [pda host = 0].
	self
		assert: [pda robot = 0].
	self
		assert: [pda interf = 0].
	self
		assert: [pda index = 0]! !

!PlayerClientTest methodsFor: 'testing' stamp: 'zz 2/4/2009 11:18'!
testDefaultHostIsLocalhost
	self
		assert: [playerClient defaultHost = #(127 0 0 1 )]! !

!PlayerClientTest methodsFor: 'testing' stamp: 'zz 2/4/2009 11:18'!
testDefaultPortIs6665
	self
		assert: [playerClient defaultPort == 6665]! !

!PlayerClientTest methodsFor: 'testing' stamp: 'SergeStinckwich 2/26/2009 12:00'!
testHeaderSizeIs40
	self assert: 
		[ (playerClient 
			headerWithType: playerClient playerMsgtypeReq
			subType: playerClient playerPlayerReqDevList
			size: 0) size = 40 ]! !

TestCase subclass: #XDRTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Tests'!

!XDRTest methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/26/2009 15:04'!
testConvertADoubleAsXDRDoubleAndBack
	self
		assert: [(1.0 xdrDouble doubleAt: 1 bigEndian: true)
				= 1.0].
	self
		assert: [(-1.0 xdrDouble doubleAt: 1 bigEndian: true)
				= -1.0]! !

!XDRTest methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/26/2009 15:11'!
testConvertAFloatAsXDRFloatAndBack
	self
		assert: [(1.0 xdrFloat floatAt: 1 bigEndian: true)
				= 1.0].
	self
		assert: [(-1.0 xdrFloat floatAt: 1 bigEndian: true)
				= -1.0]! !

!XDRTest methodsFor: 'as yet unclassified' stamp: 'zz 1/14/2009 17:19'!
testLessSignifiantByteOfMaxValueIs127
	self
		assert: [(2147483647 xdrInteger at: 1)
				= 127]! !

!XDRTest methodsFor: 'as yet unclassified' stamp: 'zz 1/14/2009 17:19'!
testLessSignifiantByteOfMinValueIs128
	self
		assert: [(-2147483648 xdrInteger at: 1)
				= 128]! !

!XDRTest methodsFor: 'as yet unclassified' stamp: 'zz 1/14/2009 17:19'!
testMostSignifiantByteOfMaxValueIs255
	self
		assert: [(2147483647 xdrInteger at: 4)
				= 255]! !

!XDRTest methodsFor: 'as yet unclassified' stamp: 'zz 1/14/2009 17:19'!
testMostSignifiantByteOfMaxValueIsZero
	self
		assert: [(-2147483648 xdrInteger at: 4)
				= 0]! !

!XDRTest methodsFor: 'as yet unclassified' stamp: 'zz 2/5/2009 14:03'!
testXDRByteOneIsCodedAsOneOnTheLastByte
	self
		assert: [(1 xdrByte at: 4)
				= 1]! !

!XDRTest methodsFor: 'as yet unclassified' stamp: 'zz 2/5/2009 13:43'!
testXDRDoubleIsCodedOnEightBytes
	self
		assert: [0.0 xdrDouble size = 8]! !

!XDRTest methodsFor: 'as yet unclassified' stamp: 'zz 1/14/2009 16:54'!
testXDRFalseIsCodedOnFourBytes
	self
		assert: [false xdrBool size = 4]! !

!XDRTest methodsFor: 'as yet unclassified' stamp: 'zz 1/15/2009 09:26'!
testXDRFalseLastByteIsZero
	self
		assert: [false xdrBool last = 0]! !

!XDRTest methodsFor: 'as yet unclassified' stamp: 'zz 1/14/2009 16:55'!
testXDRIntegerIsCodedOnFourBytes
	self
		assert: [0 xdrInteger size = 4]! !

!XDRTest methodsFor: 'as yet unclassified' stamp: 'zz 2/5/2009 13:43'!
testXDRShortIsCodedOnFourBytes
	self
		assert: [0 xdrShort size = 4]! !

!XDRTest methodsFor: 'as yet unclassified' stamp: 'zz 1/15/2009 14:45'!
testXDRStringLatinAlphabetLenghtIs26
	self
		assert: [('ABCDEFGHIJKLMNOPQRSTUVWXYZ' xdrString unsignedLongAt: 1 bigEndian: true)
				= 26]! !

!XDRTest methodsFor: 'as yet unclassified' stamp: 'zz 1/15/2009 14:45'!
testXDRStringSizeIsaMultipleOfFour
	self
		assert: ['Hello World' xdrString size \\ 4 = 0]! !

!XDRTest methodsFor: 'as yet unclassified' stamp: 'zz 1/14/2009 16:56'!
testXDRTrueIsCodedOnFourBytes
	self
		assert: [true xdrBool size = 4]! !

!XDRTest methodsFor: 'as yet unclassified' stamp: 'zz 1/14/2009 16:55'!
testXDRTrueLastByteIsOne
	self
		assert: [true xdrBool last = 1]! !

!True methodsFor: '*PlayerST' stamp: 'zz 1/14/2009 17:25'!
xdrBool
	^ ByteArray
		with: 0
		with: 0
		with: 0
		with: 1! !

!Number methodsFor: '*PlayerST' stamp: 'SergeStinckwich 2/26/2009 17:31'!
xdrByte
	^ ByteArray with: 0 with:0 with:0 with:self! !

!Number methodsFor: '*PlayerST' stamp: 'SergeStinckwich 2/26/2009 15:01'!
xdrDouble
	^ (ReferenceStream streamedRepresentationOf: self)
		copyFrom: 2
		to: 9! !

!Number methodsFor: '*PlayerST' stamp: 'SergeStinckwich 2/26/2009 15:12'!
xdrEnum 
	^ self xdrInteger! !

!Number methodsFor: '*PlayerST' stamp: 'SergeStinckwich 2/26/2009 16:13'!
xdrFloat
	| fltArray word |
	fltArray := FloatArray with: self.
	word := fltArray basicAt: 1.
	^ (ByteArray new: 4)
		unsignedLongAt: 1
		put: word
		bigEndian: true;
		 yourself! !

!Number methodsFor: '*PlayerST' stamp: 'SergeStinckwich 2/26/2009 15:12'!
xdrInteger
	| bytes | 
	bytes := ByteArray new: 4.
	self > 0
		ifTrue: [bytes
				at: 1
				put: (self bitShift: -24)]
		ifFalse: [bytes at: 1 put: ((self bitShift: -24) negated)].
	bytes at: 4 put: self \\ 256.
	bytes at: 3 put: (self bitShift: -8)
			\\ 256.
	bytes at: 2 put: (self bitShift: -16)
			\\ 256.
	^ bytes! !

!Number methodsFor: '*PlayerST' stamp: 'SergeStinckwich 2/26/2009 15:12'!
xdrLong
	^ self xdrInteger! !

!Number methodsFor: '*PlayerST' stamp: 'SergeStinckwich 2/26/2009 17:34'!
xdrShort
	| bytes |
	bytes := ByteArray new: 4.
	bytes
		at: 3
		put: (self bitShift: -8).
	bytes at: 4 put: self \\ 256.
	^ bytes! !

!Number methodsFor: '*PlayerST' stamp: 'SergeStinckwich 2/26/2009 15:13'!
xdrUnsignedInteger
	^ self xdrInteger ! !

!False methodsFor: '*PlayerST' stamp: 'zz 1/15/2009 14:28'!
xdrBool
	^ ByteArray
		with: 0
		with: 0
		with: 0
		with: 0! !

Object subclass: #PlayerBoundingBox2d
	instanceVariableNames: 'sw sl'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Interfaces'!

!PlayerBoundingBox2d commentStamp: 'SergeStinckwich 3/24/2009 14:50' prior: 0!
/** @brief A rectangular bounding box, used to define the size of an object */
typedef struct player_bbox2d
{
  /** Width [m] */
  double sw;
  /** Length [m] */
  double sl;
} player_bbox2d_t;
!

!PlayerBoundingBox2d methodsFor: 'accessing' stamp: 'SergeStinckwich 3/24/2009 15:04'!
sl
	^ sl! !

!PlayerBoundingBox2d methodsFor: 'accessing' stamp: 'SergeStinckwich 3/24/2009 15:04'!
sl: aFloat 
	sl := aFloat! !

!PlayerBoundingBox2d methodsFor: 'accessing' stamp: 'SergeStinckwich 3/24/2009 15:05'!
sw
	^ sw! !

!PlayerBoundingBox2d methodsFor: 'accessing' stamp: 'SergeStinckwich 3/24/2009 15:05'!
sw: aFloat 
	sw := aFloat! !

Object subclass: #PlayerClient
	instanceVariableNames: 'socket banner port host dataMode isThreaded isRunning readyPDDList devAddrList deviceList newPd readyRequestDevice'
	classVariableNames: 'InterfaceCodeStrings PlayerActarrayCode PlayerAioCode PlayerAudioCode PlayerAudiodspCode PlayerAudiomixerCode PlayerBlinkenlightCode PlayerBlobfinderCode PlayerBumperCode PlayerCameraCode PlayerDioCode PlayerEnergyCode PlayerFiducialCode PlayerGPSCode PlayerGpsCode PlayerGraphics2dCode PlayerGraphics3dCode PlayerGripperCode PlayerHealthCode PlayerImuCode PlayerIrCode PlayerJoystickCode PlayerLaserCode PlayerLimbCode PlayerLocalizeCode PlayerLogCode PlayerMapCode PlayerMcomCode PlayerNomadCode PlayerNullCode PlayerOpaqueCode PlayerPlannerCode PlayerPlayerCode PlayerPointcloud3dCode PlayerPosition1dCode PlayerPosition2dCode PlayerPosition3dCode PlayerPowerCode PlayerPtzCode PlayerRfidCode PlayerServiceAdvCode PlayerSimulationCode PlayerSonarCode PlayerSoundCode PlayerSpeechCode PlayerSpeechRecognitionCode PlayerTruthCode PlayerWaveformCode PlayerWifiCode PlayerWsnCode'
	poolDictionaries: ''
	category: 'PlayerST-Kernel'!

!PlayerClient commentStamp: 'SergeStinckwich 3/24/2009 14:51' prior: 0!
La liste des constante se trouve dans : http://java-player.svn.sourceforge.net/viewvc/java-player/javaclient2/structures/PlayerConstants.java?revision=76&view=markup
ou encore dans le code C de Player.!

!PlayerClient classSide methodsFor: 'as yet unclassified' stamp: 'zz 2/6/2009 10:15'!
initialize
	super initialize.
	self initializeInterfaceCodes.
	self initializeInterfaceCodeStrings ! !

!PlayerClient classSide methodsFor: 'as yet unclassified' stamp: 'zz 2/6/2009 10:36'!
initializeInterfaceCodeStrings
	InterfaceCodeStrings := Dictionary newFrom:{
	PlayerActarrayCode -> 'actarray'.
	PlayerAioCode -> 'aio'.
	PlayerAudioCode -> 'audio'.
	PlayerAudiodspCode -> 'audiodsp'.
	PlayerAudiomixerCode -> 'audiomixer'.
	PlayerBlinkenlightCode -> 'blikenlight'.
	PlayerBlobfinderCode -> 'blobfinder'.
	PlayerBumperCode -> 'bumper'.
	PlayerCameraCode -> 'camera'.
	PlayerEnergyCode -> 'energy'.
	PlayerDioCode -> 'dio'.
	PlayerGripperCode -> 'gripper'.
	PlayerFiducialCode -> 'fiducial'.
	PlayerGpsCode -> 'gps'.
	PlayerIrCode -> 'ir'.
	PlayerJoystickCode -> 'joystick'.
	PlayerLaserCode -> 'laser'.
	PlayerLimbCode -> 'limb'.
	PlayerLocalizeCode -> 'localize'.
	PlayerLogCode -> 'log'.
	PlayerMapCode -> 'map'.
	PlayerMcomCode -> 'mcom'.
	PlayerNomadCode -> 'nomad'.
	PlayerNullCode -> 'null'.
	PlayerOpaqueCode -> 'opaque'.
	PlayerPlannerCode -> 'planner'.
	PlayerPlayerCode -> 'player'.
	PlayerPosition1dCode -> 'position1d'.
	PlayerPosition2dCode -> 'position2d'.
	PlayerPosition3dCode -> 'position3d'.
	PlayerPowerCode -> 'power'.
	PlayerPtzCode -> 'ptz'.
	PlayerRfidCode -> 'rfid'.
	PlayerServiceAdvCode -> 'service_adv'.
	PlayerSimulationCode -> 'simulation'.
	PlayerSonarCode -> 'sonar'.
	PlayerSoundCode -> 'sound'.
	PlayerSpeechCode -> 'speech'.
	PlayerSpeechRecognitionCode -> 'speech_recognition'.
	PlayerTruthCode -> 'truth'.
	PlayerWaveformCode -> 'waveform'.
	PlayerWifiCode -> 'wifi'.
	PlayerWsnCode -> 'wsn'.
	PlayerGraphics2dCode -> 'graphics2d'.
	PlayerGraphics3dCode -> 'graphics3d'.
	PlayerHealthCode -> 'health'.
	PlayerImuCode -> 'imu'.
	PlayerPointcloud3dCode -> 'pointcloud3d'}! !

!PlayerClient classSide methodsFor: 'as yet unclassified' stamp: 'zz 2/5/2009 18:16'!
initializeInterfaceCodes
	PlayerNullCode := 256.
	"// /dev/null analogue"
	PlayerPlayerCode := 1.
	"// the server itself"
	PlayerPowerCode := 2.
	"// power subsystem"
	PlayerGripperCode := 3.
	"// gripper"
	PlayerPosition2dCode := 4.
	"// device that moves"
	PlayerSonarCode := 5.
	"// fixed range-finder"
	PlayerLaserCode := 6.
	"// scanning range-finder"
	PlayerBlobfinderCode := 7.
	"// visual blobfinder"
	PlayerPtzCode := 8.
	"// pan-tilt-zoom unit"
	PlayerAudioCode := 9.
	"// audio I/O"
	PlayerFiducialCode := 10.
	"// fiducial detector"
	PlayerSpeechCode := 12.
	"// speech I/O"
	PlayerGpsCode := 13.
	"// GPS unit"
	PlayerBumperCode := 14.
	"// bumper array"
	PlayerTruthCode := 15.
	"// ground-truth (Stage)"
	PlayerDioCode := 20.
	"// digital I/O"
	PlayerAioCode := 21.
	"// analog I/O"
	PlayerIrCode := 22.
	"// IR array"
	PlayerWifiCode := 23.
	"// wifi card status"
	PlayerWaveformCode := 24.
	"// fetch raw waveforms"
	PlayerLocalizeCode := 25.
	"// localization"
	PlayerMcomCode := 26.
	"// multicoms"
	PlayerSoundCode := 27.
	"// sound file playback"
	PlayerAudiodspCode := 28.
	"// audio dsp I/O"
	PlayerAudiomixerCode := 29.
	"// audio I/O"
	PlayerPosition3dCode := 30.
	"// 3-D position"
	PlayerSimulationCode := 31.
	"// simulators"
	PlayerServiceAdvCode := 32.
	"// LAN advertisement"
	PlayerBlinkenlightCode := 33.
	"// blinking lights"
	PlayerNomadCode := 34.
	"// Nomad robot"
	PlayerCameraCode := 40.
	"// camera device(gazebo)"
	PlayerMapCode := 42.
	"// get a map"
	PlayerPlannerCode := 44.
	"// 2D motion planner"
	PlayerLogCode := 45.
	"// log R/W control"
	PlayerEnergyCode := 46.
	"// energy charging"
	PlayerJoystickCode := 49.
	"// Joystick"
	PlayerSpeechRecognitionCode := 50.
	"speech I/O"
	PlayerOpaqueCode := 51.
	"plugin interface"
	PlayerPosition1dCode := 52.
	"1-D position"
	PlayerActarrayCode := 53.
	"Actuator Array interface"
	PlayerLimbCode := 54.
	"Limb interface"
	PlayerGraphics2dCode := 55.
	"Graphics2D interface"
	PlayerRfidCode := 56.
	"RFID reader interface"
	PlayerWsnCode := 57.
	"WSN interface"
	PlayerGraphics3dCode := 58.
	"Graphics3D interface"
	PlayerHealthCode := 59.
	"Statgrab Health interface"
	PlayerImuCode := 60.
	"Inertial Measurement Unit interface"
	PlayerPointcloud3dCode := 61
	"3-D point cloud"
! !

!PlayerClient classSide methodsFor: 'as yet unclassified' stamp: 'zz 2/3/2009 12:04'!
ip: ipAddressArray
	^self new
		ip: ipAddressArray;
		yourself! !

!PlayerClient classSide methodsFor: 'as yet unclassified' stamp: 'zz 2/3/2009 14:10'!
ip: ipAddressArray port: aPortNumber
	^self new
		ip: ipAddressArray;
		port: aPortNumber;
		yourself! !

!PlayerClient classSide methodsFor: 'as yet unclassified' stamp: 'zz 2/6/2009 10:17'!
lookupNameFor: interfaceCode 
	^ InterfaceCodeStrings at: interfaceCode! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'zz 2/3/2009 12:11'!
banner
	^ banner! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'zz 2/3/2009 14:08'!
banner: aByteArray 
	banner := aByteArray! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/24/2009 14:18'!
closeSocket
	socket
		ifNotNil: [socket closeAndDestroy]! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/12/2009 15:30'!
decodeDevAddressFrom: aByteArray 
	| p |
	p := PlayerDevAddr new.
	p
		host: (aByteArray longAt: 1 bigEndian: true).
	p
		robot: (aByteArray longAt: 5 bigEndian: true).
	p
		interf: (aByteArray longAt: 9 bigEndian: true).
	p
		index: (aByteArray longAt: 13 bigEndian: true).
	^ p! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/26/2009 14:00'!
defaultHost
	^ #(127 0 0 1)! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'zz 2/3/2009 13:59'!
defaultPort
	^ 6665! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'zz 2/5/2009 18:25'!
devAddrList
	^ devAddrList! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/24/2009 14:18'!
finalize
	super finalize.
	self closeSocket! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/24/2009 10:36'!
getRequestedDeviceWithInterfaceCode: aNumber index: anIndex 
	"Returns an object of type PlayerDevice containing the requested device."
	| pda |
	[readyRequestDevice] whileFalse.
	pda := newPd deviceAddress.
	pda interf = aNumber & (pda index = anIndex)
		ifTrue: [^ newPd].
	^ nil! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/24/2009 13:22'!
handleRequestsDeviceWithHeader: aPlayerMsgHeader 
	"Calls the device's handleResponse method in case of a REQ/REP."
	| currAddr devAddr |
	devAddr := aPlayerMsgHeader deviceAddress.
	deviceList
		do: [:each | 
			currAddr := each deviceAddress.
			currAddr host = devAddr host & (currAddr index = devAddr index) & (currAddr robot = devAddr robot) & (currAddr interf = devAddr interf)
				ifTrue: [each handleResponse: aPlayerMsgHeader]]! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'zz 2/5/2009 16:48'!
headerWithType: aTypeNumber subType: aSubTypeNumber size: aSizeNumber 
	| devAddr xdr timeStamp |
	devAddr := PlayerDevAddr new.
	devAddr host: 0.
	devAddr robot: 0.
	devAddr interf: PlayerPlayerCode.
	devAddr index: 0.
	timeStamp := (DateAndTime now - DateAndTime epoch) asSeconds asFloat.
	xdr := devAddr host xdrInteger.
	xdr := xdr , devAddr robot xdrInteger.
	xdr := xdr , devAddr interf xdrShort.
	xdr := xdr , devAddr index xdrShort.
	xdr := xdr , aTypeNumber xdrByte.
	xdr := xdr , aSubTypeNumber xdrByte.
	xdr := xdr , timeStamp xdrDouble.
	xdr := xdr , 0 xdrInteger.
	xdr := xdr , aSizeNumber xdrInteger.
	^ xdr! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'zz 2/3/2009 15:07'!
host
	host isNil
		ifTrue: [host := self defaultHost].
	^ host! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'zz 2/3/2009 15:07'!
host: anHost 
	host := anHost! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/16/2009 14:56'!
initialize
	super initialize.
	isThreaded := false.
	dataMode := self playerDataModePush.
	readyPDDList := false.
	deviceList := Set new.
	self banner: (ByteArray new: self playerMsgHeadSize)
	
	! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'zz 2/5/2009 10:21'!
isReadyPDDList
	"Check to see if the Player server replied with a
	PLAYER_PLAYER_REQ_DEVLIST successfully. Return true if the
	PLAYER_PLAYER_REQ_DEVLIST occured, false otherwise"
	readyPDDList
		ifTrue: [readyPDDList := false.
			^ true].
	^ false! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'zz 2/4/2009 23:01'!
open
	Socket initializeNetwork.
	socket := Socket newTCP.
	socket connectTo: self host asByteArray port: self port.
	socket waitForConnectionFor: 10.
	socket receiveDataInto: self banner! !

!PlayerClient methodsFor: 'constants' stamp: 'SergeStinckwich 2/24/2009 13:23'!
playerCloseMode
	^ 2! !

!PlayerClient methodsFor: 'constants - data delivery modes' stamp: 'zz 2/5/2009 10:07'!
playerDataModePull
	"Data delivery mode: Only on request, send data from all subscribed
	devices. A PLAYER_MSGTYPE_SYNCH packet follows each set of data.
	Request should be made automatically by client libraries when they
	begin reading."
	^ 2! !

!PlayerClient methodsFor: 'constants - data delivery modes' stamp: 'zz 2/5/2009 09:59'!
playerDataModePush
	"Data delivery mode: Send data from all subscribed devices all the time
	(i.e. when it's ready on the server)."
	^ 1! !

!PlayerClient methodsFor: 'constants' stamp: 'SergeStinckwich 2/24/2009 13:23'!
playerErrorMode
	^ 3! !

!PlayerClient methodsFor: 'constants' stamp: 'zz 2/3/2009 13:53'!
playerMsgHeadSize
	^ 32! !

!PlayerClient methodsFor: 'constants - player message types' stamp: 'SergeStinckwich 2/26/2009 11:59'!
playerMsgtypeCmd
	"A command message. Such messages are asynchronously published to
	devices, and are usually used to change some aspect of the device's
	state."
	^ 2! !

!PlayerClient methodsFor: 'constants - player message types' stamp: 'SergeStinckwich 2/26/2009 12:00'!
playerMsgtypeData
	"A data message. Such messages are asynchronously published from
	devices, and are usually used to reflect some part of the device's
	state."
	^ 1! !

!PlayerClient methodsFor: 'constants - player message types' stamp: 'SergeStinckwich 2/26/2009 12:00'!
playerMsgtypeReq
	"A request message. Such messages are published synchronously to
	devices, usually to get or set some aspect of the device's state that is
	not available in data or command messages. Every request message gets
	a response message (either PLAYER_MSGTYPE_RESP_ACK or
	PLAYER_MSGTYPE_RESP_NACK)"
	^ 3! !

!PlayerClient methodsFor: 'constants - player message types' stamp: 'SergeStinckwich 2/26/2009 12:00'!
playerMsgtypeRespAck
	"A positive response message. Such messages are published in response to
	a PLAYER_MSGTYPE_REQ. This message indicates that the underlying
	driver received, interpreted, and processed the request. Any requested
	data is in the body of this response message"
	^ 4! !

!PlayerClient methodsFor: 'constants - player message types' stamp: 'zz 2/5/2009 10:59'!
playerMsgtypeSync
	"A synch message. Deprecate this message type?"
	^ 5! !

!PlayerClient methodsFor: 'constants' stamp: 'SergeStinckwich 3/27/2009 23:06'!
playerOpenMode

^1! !

!PlayerClient methodsFor: 'constants - request subtypes' stamp: 'zz 2/5/2009 10:15'!
playerPlayerReqAddReplaceRule
	^ 10! !

!PlayerClient methodsFor: 'constants - request subtypes' stamp: 'zz 2/5/2009 10:14'!
playerPlayerReqAuth
	^ 7! !

!PlayerClient methodsFor: 'constants - request subtypes' stamp: 'zz 2/5/2009 10:12'!
playerPlayerReqData
	^ 4! !

!PlayerClient methodsFor: 'constants - request subtypes' stamp: 'zz 2/5/2009 10:14'!
playerPlayerReqDatafreq
	^ 6! !

!PlayerClient methodsFor: 'constants - request subtypes' stamp: 'zz 2/5/2009 10:13'!
playerPlayerReqDatamode
	^ 5! !

!PlayerClient methodsFor: 'constants' stamp: 'SergeStinckwich 2/24/2009 09:14'!
playerPlayerReqDev
	^ 3! !

!PlayerClient methodsFor: 'constants - request subtypes' stamp: 'zz 2/4/2009 22:50'!
playerPlayerReqDevList
	^ 1! !

!PlayerClient methodsFor: 'constants - request subtypes' stamp: 'zz 2/5/2009 10:12'!
playerPlayerReqDriverInfo
	^ 2! !

!PlayerClient methodsFor: 'constants - request subtypes' stamp: 'zz 2/5/2009 10:15'!
playerPlayerReqIdent
	^ 9! !

!PlayerClient methodsFor: 'constants - request subtypes' stamp: 'zz 2/5/2009 10:15'!
playerPlayerReqNameservice
	^ 8! !

!PlayerClient methodsFor: 'constants' stamp: 'zz 2/6/2009 10:46'!
playerxdrDevaddrSize
	^ 16! !

!PlayerClient methodsFor: 'constants' stamp: 'zz 2/5/2009 14:06'!
playerxdrMsghdrSize
	^ 40! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'zz 2/3/2009 14:12'!
port
	port isNil
		ifTrue: [port := self defaultPort].
	^ port! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'zz 2/3/2009 14:11'!
port: aPortNumber 
	port := aPortNumber! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/26/2009 14:20'!
read
	"Read the Player server replies.
	return the message type code
	"
	^ self readInterface: 0 index: 0! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'zz 2/5/2009 11:04'!
readAll
	"Read the Player server replies in non-threaded mode."
	| msgType |
	msgType := 0.
	isThreaded
		ifTrue: [^ msgType].
	dataMode = self playerDataModePull
		ifTrue: [self requestData.
			[msgType ~= self playerMsgTypeSynch]
				whileTrue: [msgType := self readInterface: 0 index: 0].
			^ self playerMsgTypeSynch].
	msgType := self readInterface: 0 index: 0.
	^ msgType! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/24/2009 13:54'!
readDataDevice: aPlayerMsgHeader 
	"Calls the device's readDataWithPlayerMsgHeader: method."
	| devAddr currAddr |
	devAddr := aPlayerMsgHeader deviceAddress.
	deviceList
		do: [:each | 
			currAddr := each deviceAddress.
			currAddr host = devAddr host & (currAddr index = devAddr index) & (currAddr robot = devAddr robot) & (currAddr interf = devAddr interf)
				ifTrue: [each readDataWithPlayerMsgHeader: aPlayerMsgHeader]]! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/24/2009 15:48'!
readHeader
	"Reads the Player message header from the network."
	| deviceAddress xdr header |
	deviceAddress := PlayerDevAddr new.
	header := PlayerMsgHdr new.
	xdr := ByteArray new: self playerxdrMsghdrSize.
	socket receiveDataInto: xdr.
	"Decode the player_devaddr"
	deviceAddress
		host: (xdr longAt: 1 bigEndian: true).
	deviceAddress
		robot: (xdr longAt: 5 bigEndian: true).
	deviceAddress interf: (xdr at: 11)
			* 256
			+ (xdr at: 12).
	deviceAddress index: (xdr at: 15)
			* 256
			+ (xdr at: 16).
	header deviceAddress: deviceAddress.
	header
		type: (xdr at: 20).
	header
		subType: (xdr at: 24).
	header timeStamp: (xdr doubleAt: 25 bigEndian: true).
	header seq: (xdr at: 33)
			* 256
			+ (xdr at: 34) * 256
			+ (xdr at: 35) * 256
			+ (xdr at: 36).
	header size: (xdr at: 37)
			* 256
			+ (xdr at: 38) * 256
			+ (xdr at: 39) * 256
			+ (xdr at: 40).
	^ header! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/26/2009 18:01'!
readInterface: anInterface index: indexNumber 
	"Read the Player server replies."
	| header devAddr access driverNameCount |
	header := self readHeader.
	header type = self playerMsgtypeData ifTrue: 
		[ Transcript
			show: '[PlayerClient][Debug]: Data for ' , header deviceAddress interf asString , ':' , header deviceAddress index asString;
			cr.
		header deviceAddress interf isZero ifTrue: [ ^ nil ].
		header deviceAddress interf ~= PlayerPlayerCode ifTrue: [ self readDataDevice: header ] ].
	header type = self playerMsgtypeRespAck ifTrue: 
		[ header deviceAddress interf ~= PlayerPlayerCode 
			ifTrue: [ self handleRequestsDeviceWithHeader: header ]
			ifFalse: 
				[ header subType = self playerPlayerReqDevList ifTrue: 
					[ "Read devices_count and array count (4+4)"
					| buffer deviceNumber |
					buffer := ByteArray new: 8.
					socket receiveDataInto: buffer.
					deviceNumber := buffer 
						longAt: 1
						bigEndian: true.
					devAddrList := Set new.
					1 
						to: deviceNumber
						do: 
							[ :each | 
							| d |
							d := ByteArray new: self playerxdrDevaddrSize.
							socket receiveDataInto: d.
							devAddrList add: (self decodeDevAddressFrom: d) ] ].
				header subType = self playerPlayerReqDev ifTrue: 
					[ | buffer |
					buffer := ByteArray new: self playerxdrDevaddrSize.
					socket receiveDataInto: buffer.
					devAddr := self decodeDevAddressFrom: buffer.
					buffer := ByteArray new: 12.
					socket receiveDataInto: buffer.
					access := buffer at: 4.
					driverNameCount := buffer 
						longAt: 5
						bigEndian: true.
					buffer := ByteArray new: driverNameCount.
					socket receiveDataInto: buffer.
					(driverNameCount isDivisibleBy: 4) ifFalse: [ socket receiveDataInto: (ByteArray new: (4 - driverNameCount) \\ 4) ].
					Transcript
						show: '[PlayerClient][Debug]: Got response: ' , devAddr interf asString , ':' , devAddr index asString , '(' , buffer asString , ')' , ' size of payload:' , header size asString;
						cr.
					newPd := self 
						requestSatisfyWithPlayerAddress: devAddr
						accessMode: access
						driverName: buffer asString.
					readyRequestDevice := true ] ] ].
	^ header type! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/26/2009 12:00'!
requestData
	"Configuration request: Get data. When the server is in a
	PLAYER_DATAMODE_PULL data delivery mode, the client can request a
	single round of data by sending a PLAYER_PLAYER_REQ_DATA request."
	| xdr |
	xdr := self 
		headerWithType: self playerMsgtypeReq
		subType: self playerPlayerReqData
		size: 0.
	socket sendData: xdr! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/26/2009 12:00'!
requestDeviceAccessWithInterfaceCode: interfaceCode index: aNumber access: accessMode 
	| devAddr xdr header result |
	devAddr := PlayerDevAddr new.
	devAddr
		robot: 0;
		host: 0;
		interf: interfaceCode;
		index: aNumber.
	xdr := devAddr host xdrInteger.
	xdr := xdr , devAddr robot xdrInteger.
	xdr := xdr , devAddr interf xdrShort.
	xdr := xdr , devAddr index xdrShort.
	xdr := xdr , accessMode xdrByte.
	xdr := xdr , 0 xdrInteger.
	xdr := xdr , 0 xdrInteger.
	header := self 
		headerWithType: self playerMsgtypeReq
		subType: self playerPlayerReqDev
		size: xdr size.
	socket sendData: header , xdr.
	
	[ result := self 
		readInterface: interfaceCode
		index: aNumber.
	result ~= self playerMsgtypeRespAck ] whileTrue.
	^ self 
		getRequestedDeviceWithInterfaceCode: interfaceCode
		index: aNumber! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/26/2009 12:00'!
requestDeviceList
	"Request/reply: Get the list of available devices.
	It's useful for applications such as viewer programs and test suites
	that tailor behave differently depending on which devices are
	available. To request the list, send a null PLAYER_PLAYER_REQ_DEVLIST
	"
	| deviceCount arrayCount xdr size header |
	deviceCount := 0 xdrInteger.
	arrayCount := 0 xdrInteger.
	xdr := deviceCount , arrayCount.
	size := xdr size.
	header := self 
		headerWithType: self playerMsgtypeReq
		subType: self playerPlayerReqDevList
		size: size.
	socket sendData: header , xdr! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/16/2009 14:48'!
requestInterfacePosition2DWithIndex: aNumber access: accessMode 
	^self
		requestDeviceAccessWithInterfaceCode: PlayerPosition2dCode
		index: aNumber
		access: accessMode! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/27/2009 16:01'!
requestInterfaceSonarWithIndex: aNumber access: accessMode 
	^ self
		requestDeviceAccessWithInterfaceCode: PlayerSonarCode
		index: aNumber
		access: accessMode! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 11:02'!
requestInterfaceWifiWithIndex: aNumber access: accessMode 
	^ self
		requestDeviceAccessWithInterfaceCode: PlayerWifiCode
		index: aNumber
		access: accessMode! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 11:00'!
requestSatisfyWithPlayerAddress: aPlayerDevAddr accessMode: mode driverName: aString 
	| newDevice |
	mode == self playerCloseMode
		ifTrue: [^ nil].
	aPlayerDevAddr interf == PlayerPosition2dCode
		ifTrue: [newDevice := PositionDevice2D new].
	aPlayerDevAddr interf == PlayerSonarCode
		ifTrue: [newDevice := SonarInterface new].
		aPlayerDevAddr interf == PlayerWifiCode 
		ifTrue: [newDevice := WifiInterface new].
	newDevice
		ifNotNil: [newDevice deviceAddress: aPlayerDevAddr.
			newDevice deviceAccess: mode.
			newDevice deviceDriverName: aString.
			newDevice playerClient: self.
			deviceList add: newDevice].
	^ newDevice! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/24/2009 16:48'!
sendMessageWithType: aType subType: aSubType data: aDataStructure
! !

!PlayerClient methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/23/2009 13:53'!
socket
	(socket isNil
			or: [socket isConnected not])
		ifTrue: [self initSocket].
	^ socket! !

!PlayerClient methodsFor: 'process management' stamp: 'SergeStinckwich 2/24/2009 13:16'!
startThread
	| resultCode |
	isRunning := true.
	isThreaded := true.
	[[isThreaded]
		whileTrue: [dataMode = self playerDataModePull
				ifTrue: [self requestData.
					resultCode := self read.
					[resultCode ~= self playerMsgtypeSync & isThreaded] whileTrue]
				ifFalse: [self read]].
			isRunning := false.
			] fork! !

!PlayerClient methodsFor: 'process management' stamp: 'SergeStinckwich 2/24/2009 15:10'!
stopThread
	isThreaded := false.
"	[isRunning] whileTrue"! !

Object subclass: #PlayerClientDemo
	instanceVariableNames: 'playerClient'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Kernel'!

!PlayerClientDemo commentStamp: 'zz 2/5/2009 15:15' prior: 0!
Write inside a Transcript :

"
| demo |
demo := PlayerClientDemo new.
demo startDemo.
"!

!PlayerClientDemo methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 17:08'!
initialize
	playerClient := PlayerClient new.
	playerClient open! !

!PlayerClientDemo methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/4/2009 10:05'!
startDemo1
	"player stage-2.1.1/worlds/simple.cfg"
	Transcript cr; show: 'Server version = '; show: playerClient banner asString; cr.
	playerClient requestDeviceList.
	playerClient readAll.
	Transcript show: 'Number of devices = ';
		 show: playerClient devAddrList size;
		 cr.
	Transcript show: 'Devices = ';
		 show: playerClient devAddrList asString;
		 cr.
	playerClient closeSocket! !

!PlayerClientDemo methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/4/2009 10:05'!
startDemo2
	"player stage-2.1.1/worlds/simple.cfg"
	| position2D pp2d p |
	position2D := playerClient 
		requestInterfacePosition2DWithIndex: 0
		access: PlayerConstants playerOpenMode.
	playerClient startThread.
	p := PlayerPose2d new.
	p
		px: 0.0;
		py: 0.0;
		pa: 40.0 degreesToRadians.
	position2D 
		setVelocityWithPlayerPose: p
		state: 1.
	position2D queryGeometry.
	[ position2D isDataReady ] whileFalse.
	pp2d := position2D getData.
	Transcript
		show: 'State : x=' , pp2d pos px asString , ' y =' , pp2d pos py asString , ' a = ' , pp2d pos pa asString , ' stall = ' , pp2d stall asString;
		cr.
	playerClient stopThread.
	playerClient closeSocket! !

!PlayerClientDemo methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/4/2009 10:05'!
startDemo3
	"player stage-2.1.1/worlds/simple.cfg"
	| sonar s |
	sonar := playerClient requestInterfaceSonarWithIndex: 0 access: PlayerConstants playerOpenMode.
	playerClient startThread.
	
	[sonar isDataReady] whileFalse.
	s := sonar getData.
	Transcript show: 'Ranges count =', s rangesCount asString , ' Ranges : ', s ranges printString; cr.
	playerClient stopThread.
	playerClient closeSocket! !

!PlayerClientDemo methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/4/2009 10:06'!
startDemo4
	"player stage-2.1.1/worlds/wifi/wifi.cfg"
	| wifi w |
	wifi := playerClient requestInterfaceWifiWithIndex: 0 access: PlayerConstants playerOpenMode.
	playerClient startThread.
	wifi queryWifiIwspyAddrAdd.
	[wifi isDataReady] whileFalse.
	w := wifi getData.
	w inspect.
	playerClient stopThread.
	playerClient closeSocket! !

!PlayerClientDemo methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/4/2009 10:05'!
startDemo5
	"player stage-2.1.1/worlds/simple.cfg"
	| sonar s position turnRate speed |
	position := playerClient requestInterfacePosition2DWithIndex: 0 access: PlayerConstants playerOpenMode.
	sonar := playerClient requestInterfaceSonarWithIndex: 0 access: PlayerConstants playerOpenMode.
	playerClient startThread.
	100
		timesRepeat: [[sonar isDataReady] whileFalse.
			s := sonar getData.
			(s ranges at: 1)
					+ (s ranges at: 2) < ((s ranges at: 6)
						+ (s ranges at: 7))
				ifTrue: [turnRate := -20 degreesToRadians]
				ifFalse: [turnRate := 20 degreesToRadians].
			(s ranges at: 3)
					< 0.5
				ifTrue: [speed := 0.0]
				ifFalse: [speed := 0.1].
			position speed: speed withTurnRate: turnRate].
	playerClient stopThread.
	playerClient closeSocket! !

Object subclass: #PlayerDevAddr
	instanceVariableNames: 'host robot interf index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Kernel'!

!PlayerDevAddr commentStamp: 'zz 2/4/2009 18:00' prior: 0!
Devices are identified by 12-byte addresses of this form.
Some of the fields are transport-dependent in their intepretation (see the player_devaddr structure from player.h).
!

!PlayerDevAddr methodsFor: 'as yet unclassified' stamp: 'zz 2/4/2009 22:54'!
host
	^ host! !

!PlayerDevAddr methodsFor: 'as yet unclassified' stamp: 'zz 2/4/2009 22:26'!
host: anHost 
	host := anHost! !

!PlayerDevAddr methodsFor: 'as yet unclassified' stamp: 'zz 2/4/2009 22:57'!
index
	^ index! !

!PlayerDevAddr methodsFor: 'as yet unclassified' stamp: 'zz 2/4/2009 22:57'!
index: indexNumber 
	index := indexNumber! !

!PlayerDevAddr methodsFor: 'as yet unclassified' stamp: 'zz 2/4/2009 22:54'!
interf
	^ interf! !

!PlayerDevAddr methodsFor: 'as yet unclassified' stamp: 'zz 2/4/2009 22:55'!
interf: interface 
	interf := interface! !

!PlayerDevAddr methodsFor: 'as yet unclassified' stamp: 'zz 2/6/2009 10:17'!
printOn: aStream 
	aStream nextPutAll: host asString;
		 nextPutAll: ':';
		 nextPutAll: robot asString;
		 nextPutAll: ':';
		 nextPutAll: (PlayerClient lookupNameFor: interf);
		 nextPutAll: ':';
		 nextPutAll: index asString! !

!PlayerDevAddr methodsFor: 'as yet unclassified' stamp: 'zz 2/4/2009 22:54'!
robot
	^ robot! !

!PlayerDevAddr methodsFor: 'as yet unclassified' stamp: 'zz 2/4/2009 22:54'!
robot: aRobot 
	robot := aRobot! !

Object subclass: #PlayerDevice
	instanceVariableNames: 'deviceAddress deviceAccess deviceDriverName playerClient timeStamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Kernel'!

!PlayerDevice commentStamp: 'zz 2/6/2009 10:40' prior: 0!
Abstract class for all Player interfaces.!

PlayerDevice subclass: #AbstractPositionDevice
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Interfaces'!

!AbstractPositionDevice methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/12/2009 16:27'!
speed: aSpeedNumber withTurnRate: aTurnRateNumber 
	self subclassResponsibility! !

!AbstractPositionDevice methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/12/2009 14:45'!
x
	self subclassResponsibility! !

!AbstractPositionDevice methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/12/2009 14:46'!
y
	self subclassResponsibility! !

!AbstractPositionDevice methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/12/2009 14:46'!
yaw
	self subclassResponsibility! !

AbstractPositionDevice subclass: #PositionDevice2D
	instanceVariableNames: 'pp2ddata pp2dgeom readyPp2ddata readyPp2dgeom'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Interfaces'!

!PositionDevice2D commentStamp: 'zz 2/6/2009 11:07' prior: 0!
The position2d interface is used to control a mobile robot bases in 2D.!

!PositionDevice2D methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/24/2009 13:35'!
getData
	"Get the Position2D data."
	^ pp2ddata! !

!PositionDevice2D methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 17:54'!
getGeom
	^ pp2dgeom! !

!PositionDevice2D methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/26/2009 17:49'!
handleResponse: aPlayermsgHdr 
	"Handle acknowledgement response messages."
	aPlayermsgHdr subType = self playerPosition2dReqGetGeom
		ifTrue: [self readGeom]! !

!PositionDevice2D methodsFor: 'initialization' stamp: 'SergeStinckwich 3/24/2009 15:20'!
initialize
	readyPp2ddata := false.
	readyPp2dgeom := false! !

!PositionDevice2D methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/24/2009 13:32'!
isDataReady
	readyPp2ddata
		ifTrue: [readyPp2ddata := false.
			^ true].
	^ false! !

!PositionDevice2D methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 17:53'!
isGeomReady
	readyPp2dgeom
		ifTrue: [readyPp2dgeom := false.
			^ true].
	^ false! !

!PositionDevice2D methodsFor: 'constants' stamp: 'zz 2/6/2009 14:02'!
playerPosition2dCmdCar
	^ 3! !

!PositionDevice2D methodsFor: 'constants' stamp: 'SergeStinckwich 2/19/2009 15:14'!
playerPosition2dCmdPos
	^ 2! !

!PositionDevice2D methodsFor: 'constants' stamp: 'SergeStinckwich 2/26/2009 16:51'!
playerPosition2dCmdVel
	^ 1! !

!PositionDevice2D methodsFor: 'constants' stamp: 'SergeStinckwich 3/24/2009 12:14'!
playerPosition2dDataGeom
	^ 2! !

!PositionDevice2D methodsFor: 'constants' stamp: 'zz 2/6/2009 14:02'!
playerPosition2dDataState
	^ 1! !

!PositionDevice2D methodsFor: 'constants' stamp: 'SergeStinckwich 3/24/2009 12:10'!
playerPosition2dReqGetGeom
	^ 1! !

!PositionDevice2D methodsFor: 'constants' stamp: 'SergeStinckwich 2/19/2009 15:14'!
playerPosition2dReqMotorPower
	^ 2! !

!PositionDevice2D methodsFor: 'constants' stamp: 'SergeStinckwich 2/19/2009 15:14'!
playerPosition2dReqPositionMode
	^ 4! !

!PositionDevice2D methodsFor: 'constants' stamp: 'SergeStinckwich 2/19/2009 15:13'!
playerPosition2dReqPositionPid
	^ 8! !

!PositionDevice2D methodsFor: 'constants' stamp: 'SergeStinckwich 2/12/2009 14:48'!
playerPosition2dReqResetOdom
	^ 6! !

!PositionDevice2D methodsFor: 'constants' stamp: 'SergeStinckwich 3/24/2009 12:10'!
playerPosition2dReqSetOdom
	^ 5! !

!PositionDevice2D methodsFor: 'constants' stamp: 'SergeStinckwich 2/12/2009 14:48'!
playerPosition2dReqSpeedPid
	^ 7! !

!PositionDevice2D methodsFor: 'constants' stamp: 'SergeStinckwich 2/12/2009 14:48'!
playerPosition2dReqSpeedProf
	^ 9! !

!PositionDevice2D methodsFor: 'constants' stamp: 'SergeStinckwich 2/12/2009 14:48'!
playerPosition2dReqVelocityMode
	^ 3! !

!PositionDevice2D methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/26/2009 12:00'!
queryGeometry
	"Request/reply: Query geometry
	"
	| buffer |
	buffer := self 
		headerWithType: playerClient playerMsgtypeReq
		subType: self playerPosition2dReqGetGeom
		size: 0.
	playerClient socket sendData: buffer! !

!PositionDevice2D methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/24/2009 12:12'!
readDataWithPlayerMsgHeader: aHeader 
	"Read the position2d data values (state or geom)."
	aHeader subType = self playerPosition2dDataState
		ifTrue: [| buffer pos vel | 
			timeStamp := aHeader timeStamp.
			buffer := ByteArray new: aHeader size.
			pos := PlayerPose2d new.
			vel := PlayerPose2d new.
			playerClient socket receiveDataInto: buffer.
			pos
				px: (buffer doubleAt: 1 bigEndian: true).
			pos
				py: (buffer doubleAt: 9 bigEndian: true).
			pos
				pa: (buffer doubleAt: 17 bigEndian: true).
			vel
				px: (buffer doubleAt: 25 bigEndian: true).
			vel
				py: (buffer doubleAt: 33 bigEndian: true).
			vel
				pa: (buffer doubleAt: 41 bigEndian: true).
			pp2ddata := PlayerPosition2dData new.
			pp2ddata pos: pos.
			pp2ddata vel: vel.
			pp2ddata
				stall: (buffer at: 52).
			Transcript show: 'TimeStamp = ' , timeStamp asString , ' pos px = ' , pos px asString , ' pos py = ' , pos py asString , ' pos pa = ' , pos pa asString , ' vel px = ' , vel px asString , ' vel py = ' , vel py asString , ' vel pa = ' , vel pa asString , ' buffer =' , buffer printString;
				 cr.
			readyPp2ddata := true].
	aHeader subType = self playerPosition2dDataGeom
		ifTrue: [
			timeStamp := aHeader timeStamp.
			self readGeom.
			readyPp2dgeom := true.
			]! !

!PositionDevice2D methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/24/2009 15:11'!
readGeom
	| buffer pose boundingBox |
	buffer := ByteArray new: 12 + 8.
	playerClient socket receiveDataInto: buffer.
	pp2dgeom := PlayerPosition2dGeom new.
	pose := PlayerPose2d new.
	boundingBox := PlayerBoundingBox2d new.
	pose px: (buffer 
			floatAt: 1
			bigEndian: true).
	pose py: (buffer 
			floatAt: 5
			bigEndian: true).
	pose pa: (buffer 
			floatAt: 9
			bigEndian: true).
	pp2dgeom pose: pose.
	boundingBox width: (buffer 
			floatAt: 13
			bigEndian: true).
	boundingBox with: (buffer 
			floatAt: 17
			bigEndian: true).
	pp2dgeom size: boundingBox! !

!PositionDevice2D methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/26/2009 12:00'!
resetOdometry
	"Resets the robot's odometry to (x,y,theta) = (0,0,0)."
	| buffer |
	buffer := self 
		headerWithType: self playerMsgtypeReq
		subType: self playerPosition2dReqResetOdom
		size: 0.
	playerClient socket sendData: buffer! !

!PositionDevice2D methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/24/2009 15:26'!
setControlMode: aByte
	| buffer |
	buffer := self 
		headerWithType: self playerMsgtypeReq
		subType: self playerPosition2dReqPositionMode
		size: 4.
	playerClient socket sendData: buffer.
	playerClient socket sendData: aByte xdrInteger! !

!PositionDevice2D methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/24/2009 16:48'!
setMotorPower: aByte
	"Configuration request: Motor power.
	On some robots, the motor power can be turned on and off from
	software. Be VERY careful with this command!! You are very likely to
	start the robot running across the room at high speed with the battery charger still
	attached. @param state 0 for off, 1 for on"
	! !

!PositionDevice2D methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/24/2009 12:09'!
setOdometry: aPlayerPose 
	| header xdr |
	header := self
				headerWithType: playerClient playerMsgTypeReq
				subType: self playerPosition2dReqSetOdom
				size: 12.
	playerClient socket sendData: header.
	xdr := aPlayerPose px xdrFloat.
	xdr := xdr , aPlayerPose py xdrFloat.
	xdr := xdr , aPlayerPose pa xdrFloat.
	playerClient socket sendData: xdr! !

!PositionDevice2D methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/2/2009 16:59'!
setPositionWithPose: pos velocity: vel state: aState 
	| header xdr |
	header := self 
		headerWithType: playerClient playerMsgtypeCmd
		subType: self playerPosition2dCmdPos
		size: (6*8) + 4.
	xdr := pos px xdrDouble.
	xdr := xdr , pos py xdrDouble.
	xdr := xdr , pos pa xdrDouble.
	xdr := xdr , vel px xdrDouble.
	xdr := xdr , vel py xdrDouble.
	xdr := xdr , vel pa xdrDouble.
	xdr := xdr , aState xdrByte.
	playerClient socket sendData: header , xdr! !

!PositionDevice2D methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/26/2009 12:00'!
setVelocityControl: mode 
	| buffer |
	buffer := self 
		headerWithType: self playerMsgtypeReq
		subType: self playerPosition2dReqVelocityMode
		size: 4.
	playerClient socket sendData: buffer.
	buffer := mode xdrByte.
	playerClient socket sendData: buffer! !

!PositionDevice2D methodsFor: 'messages' stamp: 'SergeStinckwich 3/2/2009 16:56'!
setVelocityWithPlayerPose: vel state: aState 
	| header xdr |
	header := self
				headerWithType: playerClient playerMsgtypeCmd
				subType: self playerPosition2dCmdVel
				size: (3*8) + 4.
	playerClient socket sendData: header.
	xdr := vel px xdrDouble.
	xdr := xdr , vel py xdrDouble.
	xdr := xdr , vel pa xdrDouble.
	xdr := xdr , aState xdrByte.
	playerClient socket sendData: xdr! !

!PositionDevice2D methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 16:40'!
speed: aSpeedValue withTurnRate: aTurnValue 
	| pp |
	pp := PlayerPose2d new.
	pp px: aSpeedValue.
	pp py: 0.0.
	pp pa: aTurnValue.
	self setVelocityWithPlayerPose: pp state: 1! !

!PlayerDevice classSide methodsFor: 'as yet unclassified' stamp: 'zz 2/6/2009 11:16'!
withPlayerClient: aClient 
	^ super new playerClient: aClient! !

!PlayerDevice methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/23/2009 13:43'!
deviceAccess: access 
	deviceAccess := access! !

!PlayerDevice methodsFor: 'as yet unclassified' stamp: 'zz 2/6/2009 10:43'!
deviceAddress
	^ deviceAddress! !

!PlayerDevice methodsFor: 'as yet unclassified' stamp: 'zz 2/6/2009 10:42'!
deviceAddress: aDeviceAddress 
	deviceAddress := aDeviceAddress! !

!PlayerDevice methodsFor: 'as yet unclassified' stamp: 'zz 2/6/2009 10:43'!
deviceDriverName
	^ deviceDriverName! !

!PlayerDevice methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/23/2009 13:42'!
deviceDriverName: aString
	deviceDriverName := aString! !

!PlayerDevice methodsFor: 'as yet unclassified' stamp: 'zz 2/6/2009 10:42'!
handleResponse: aPlayerMsgHdr 
	self subclassResponsibility! !

!PlayerDevice methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/26/2009 16:53'!
headerWithType: aTypeNumber subType: aSubTypeNumber size: aSizeNumber 
	| xdr timeStamp |
	timeStamp := (DateAndTime now - DateAndTime epoch) asSeconds asFloat.
	xdr := deviceAddress host xdrInteger.
	xdr := xdr , deviceAddress robot xdrInteger.
	xdr := xdr , deviceAddress interf xdrShort.
	xdr := xdr , deviceAddress index xdrShort.
	xdr := xdr , aTypeNumber xdrByte.
	xdr := xdr , aSubTypeNumber xdrByte.
	xdr := xdr , timeStamp xdrDouble.
	xdr := xdr , 0 xdrInteger.
	xdr := xdr , aSizeNumber xdrInteger.
	^ xdr! !

!PlayerDevice methodsFor: 'as yet unclassified' stamp: 'zz 2/6/2009 13:47'!
playerClient: aClient 
	playerClient := aClient! !

!PlayerDevice methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/24/2009 11:23'!
readDataWithPlayerMsgHeader: aHeader 
	self subclassResponsibility! !

PlayerDevice subclass: #SimulationInterface
	instanceVariableNames: 'psp2ddata readyPsp2ddata'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Interfaces'!

!SimulationInterface methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 17:56'!
readDataWithPlayerMsgHeader: aHeader 
	^ self! !

PlayerDevice subclass: #SonarInterface
	instanceVariableNames: 'readyPsdata psdata psgeom readyPsgeom'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Interfaces'!

!SonarInterface commentStamp: 'SergeStinckwich 3/2/2009 16:19' prior: 0!
description {
 * @brief Array of ultrasonic rangers

The @p sonar interface provides access to a collection of fixed range
sensors, such as a sonar array.
}

/** Request/reply subtype: get geometry */
message { REQ, GET_GEOM, 1, player_sonar_geom_t };
/** Request/reply subtype: power */
message { REQ, POWER, 2, player_sonar_power_config_t };

/** Data subtype: ranges */
message { DATA, RANGES, 1, player_sonar_data_t };
/** Data subtype: geometry */
message { DATA, GEOM, 2, player_sonar_geom_t };

/** @brief Data: ranges (@ref PLAYER_SONAR_DATA_RANGES)

The @p sonar interface returns up to @ref PLAYER_SONAR_MAX_SAMPLES range
readings from a robot's sonars. */
typedef struct player_sonar_data
{
  /** The number of valid range readings. */
  uint32_t ranges_count;
  /** The range readings [m] */
  float *ranges;
} player_sonar_data_t;

/** @brief Data AND Request/reply: geometry.

To query the geometry of the sonar transducers, send a null
@ref PLAYER_SONAR_REQ_GET_GEOM request.  Depending on the underlying
driver, this message can also be sent as data with the subtype
@ref PLAYER_SONAR_DATA_GEOM. */
typedef struct player_sonar_geom
{
  /** The number of valid poses. */
  uint32_t poses_count;
  /** Pose of each sonar, in robot cs */
  player_pose3d_t *poses;
} player_sonar_geom_t;

/** @brief Request/reply: Sonar power.

On some robots, the sonars can be turned on and off from software.
To do so, send a @ref PLAYER_SONAR_REQ_POWER request.  Null response. */
typedef struct player_sonar_power_config
{
  /** Turn power off TRUE or FALSE */
  uint8_t state;
} player_sonar_power_config_t;

/** @} */
!

!SonarInterface methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/27/2009 14:10'!
getData
	^ psdata! !

!SonarInterface methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/27/2009 13:58'!
getGeom
	^ psgeom! !

!SonarInterface methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/27/2009 14:07'!
handleResponse: aPlayerMsgHdr ! !

!SonarInterface methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/27/2009 14:00'!
initialize
	readyPsdata := false.
	readyPsgeom := false.! !

!SonarInterface methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/27/2009 13:57'!
isDataReady
	readyPsdata
		ifTrue: [readyPsdata := false.
			^ true].
	^ false! !

!SonarInterface methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/27/2009 14:00'!
isGeomReady
	readyPsgeom
		ifTrue: [readyPsgeom := false.
			^ true].
	^ false! !

!SonarInterface methodsFor: 'constants' stamp: 'SergeStinckwich 2/27/2009 13:55'!
playerSonarDataGeom
	^ 2! !

!SonarInterface methodsFor: 'constants' stamp: 'SergeStinckwich 2/27/2009 13:52'!
playerSonarDataRanges
	^ 1! !

!SonarInterface methodsFor: 'constants' stamp: 'SergeStinckwich 2/27/2009 13:52'!
playerSonarMaxSamples
	^ 64! !

!SonarInterface methodsFor: 'constants' stamp: 'SergeStinckwich 2/27/2009 13:54'!
playerSonarReqGetGeom
	^ 1! !

!SonarInterface methodsFor: 'constants' stamp: 'SergeStinckwich 2/27/2009 13:54'!
playerSonarReqPower
	^ 2! !

!SonarInterface methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/20/2009 16:14'!
powerSonarOff
	self setPowerSonar: 0! !

!SonarInterface methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/20/2009 16:14'!
powerSonarOn
	self setPowerSonar: 1! !

!SonarInterface methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 13:49'!
queryGeometry
	"Request/reply: Query geometry"
	| buffer |
	buffer := self
				headerWithType: playerClient playerMsgtypeReq
				subType: self playerSonarReqGetGeom
				size: 0.
	playerClient socket sendData: buffer! !

!SonarInterface methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 13:42'!
readDataWithPlayerMsgHeader: aHeader 
	"Read the sonar values."
	| buffer rangesCount ranges |
	aHeader subType = self playerSonarDataRanges
		ifTrue: [timeStamp := aHeader timeStamp.
			buffer := ByteArray new: aHeader size.
			playerClient socket receiveDataInto: buffer.
			rangesCount := buffer longAt: 1 bigEndian: true.
			ranges := Array new: rangesCount.
			2
				to: rangesCount + 1
				do: [:i | ranges
						at: i - 1
						put: (buffer floatAt: i - 1 * 4 + 1 + 4 bigEndian: true)].
			psdata := PlayerSonarData new.
			psdata ranges: ranges.
			readyPsdata := true]! !

!SonarInterface methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/20/2009 16:14'!
setPowerSonar: aState 
	"On some robots, the sonars can be turned on and off from software.
	To do so, send a PLAYER_SONAR_REQ_POWER request.
	See the player_sonar_power_config structure from player.h
	state turn power off (0) or on (>0)"
	| buffer |
	buffer := self
				headerWithType: playerClient playerMsgtypeReq
				subType: self playerSonarReqPower
				size: 4.
	playerClient socket sendData: buffer.
	playerClient socket sendData: aState xdrByte! !

PlayerDevice subclass: #WifiInterface
	instanceVariableNames: 'pwdata readyPwdata'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Interfaces'!

!WifiInterface methodsFor: 'accessing' stamp: 'SergeStinckwich 2/27/2009 17:05'!
getData
	^ pwdata! !

!WifiInterface methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/2/2009 14:30'!
handleResponse: aPlayerMsgHdr 
	"Handle acknowledgement response messages
	"
	aPlayerMsgHdr subType = self playerWifiMac
		ifTrue: [^ self]! !

!WifiInterface methodsFor: 'initialization' stamp: 'SergeStinckwich 3/3/2009 11:06'!
initialize
	readyPwdata := false! !

!WifiInterface methodsFor: 'testing' stamp: 'SergeStinckwich 2/27/2009 17:01'!
isDataReady
	readyPwdata  
		ifTrue: [readyPwdata  := false.
			^ true].
	^ false! !

!WifiInterface methodsFor: 'constants' stamp: 'SergeStinckwich 3/2/2009 18:08'!
playerWifiDataState
	^ 1! !

!WifiInterface methodsFor: 'constants' stamp: 'SergeStinckwich 3/2/2009 18:04'!
playerWifiIwspyAdd
	^ 2! !

!WifiInterface methodsFor: 'constants' stamp: 'SergeStinckwich 3/2/2009 18:05'!
playerWifiMac
	^ 1! !

!WifiInterface methodsFor: 'constants' stamp: 'SergeStinckwich 3/2/2009 18:06'!
playerWifiMaxLinks
	^ 32! !

!WifiInterface methodsFor: 'constants' stamp: 'SergeStinckwich 3/2/2009 18:06'!
playerWifiQualDbm
	^ 1! !

!WifiInterface methodsFor: 'constants' stamp: 'SergeStinckwich 3/2/2009 18:07'!
playerWifiQualRel
	^ 2! !

!WifiInterface methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/2/2009 14:27'!
queryWifiIwspyAddrAdd
	| header |
	header := self
				headerWithType: playerClient playerMsgtypeReq
				subType: self playerWifiIwspyAdd
				size: 32 * 4.
	playerClient socket sendData: header! !

!WifiInterface methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 11:37'!
queryWifiMac
	| header |
	header := self
				headerWithType: playerClient playerMsgtypeReq 
				subType: self playerWifiMac
				size: 0.
	playerClient socket sendData: header! !

!WifiInterface methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 14:18'!
readDataWithPlayerMsgHeader: aHeader 
	| buffer linksCount pwl macCounts ipCount next essidCount pwls |
	aHeader subType = self playerWifiDataState
		ifTrue: [
			pwdata := PlayerWifiData new.
			timeStamp := aHeader timeStamp.
			buffer := ByteArray new: aHeader size.
			playerClient socket receiveDataInto: buffer.
			linksCount := buffer longAt:1 bigEndian: true.
			pwls := Set new.
			linksCount
				timesRepeat: [pwl := PlayerWifiLink new.
					macCounts := buffer longAt: 5 bigEndian: true.
					next := 5 + macCounts.
					pwl
						mac: (buffer copyFrom: 5 to: next).
					ipCount := buffer longAt: next + 1 bigEndian: true.
					next := next + 5.
					pwl
						ip: (buffer copyFrom: next to: next + ipCount).
					essidCount := buffer longAt: next + ipCount + 1 bigEndian: true.
					next := next + ipCount + 5.
					pwl
						essid: (buffer copyFrom: next to: next + essidCount).
					next := next + essidCount + 5.
					pwl
						mode: (buffer longAt: next bigEndian: true).
					pwl
						freq: (buffer longAt: next + 5 bigEndian: true).
					pwl
						encrypt: (buffer longAt: next + 9 bigEndian: true).
					pwl
						qual: (buffer longAt: next + 13 bigEndian: true).
					pwl
						level: (buffer longAt: next + 17 bigEndian: true).
					pwl
						noise: (buffer longAt: next + 22 bigEndian: true).
					pwls add: pwl].
			pwdata links: pwls.
			buffer := ByteArray new: 7 * 4.
			pwdata
				throughput: (buffer longAt: 1 bigEndian: true).
			pwdata
				bitrate: (buffer longAt: 5 bigEndian: true).
			pwdata
				mode: (buffer longAt: 9 bigEndian: true).
			pwdata
				qualType: (buffer longAt: 13 bigEndian: true).
			pwdata
				maxqual: (buffer longAt: 17 bigEndian: true).
			pwdata
				maxlevel: (buffer longAt: 21 bigEndian: true).
			pwdata
				maxnoise: (buffer longAt: 25 bigEndian: true).
			buffer := ByteArray new: 32.
			pwdata ap: buffer.
			readyPwdata := true.
			]! !

Object subclass: #PlayerMsgHdr
	instanceVariableNames: 'deviceAddress type timeStamp seq size subType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Kernel'!

!PlayerMsgHdr methodsFor: 'as yet unclassified' stamp: 'zz 2/5/2009 10:46'!
deviceAddress
	^ deviceAddress! !

!PlayerMsgHdr methodsFor: 'as yet unclassified' stamp: 'zz 2/5/2009 10:46'!
deviceAddress: aDeviceAddress 
	deviceAddress := aDeviceAddress! !

!PlayerMsgHdr methodsFor: 'as yet unclassified' stamp: 'zz 2/5/2009 14:24'!
seq
	^ seq! !

!PlayerMsgHdr methodsFor: 'as yet unclassified' stamp: 'zz 2/5/2009 14:24'!
seq: aNumber 
	seq := aNumber! !

!PlayerMsgHdr methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/24/2009 11:18'!
size
	^ size! !

!PlayerMsgHdr methodsFor: 'as yet unclassified' stamp: 'zz 2/5/2009 14:24'!
size: aNumber 
	size := aNumber! !

!PlayerMsgHdr methodsFor: 'as yet unclassified' stamp: 'zz 2/5/2009 14:15'!
subType
	^ subType! !

!PlayerMsgHdr methodsFor: 'as yet unclassified' stamp: 'zz 2/5/2009 14:24'!
subType: aSubTypeNumber 
	subType := aSubTypeNumber! !

!PlayerMsgHdr methodsFor: 'as yet unclassified' stamp: 'zz 2/5/2009 14:25'!
timeStamp
	^ timeStamp! !

!PlayerMsgHdr methodsFor: 'as yet unclassified' stamp: 'zz 2/5/2009 14:48'!
timeStamp: aNumber 
	timeStamp := aNumber! !

!PlayerMsgHdr methodsFor: 'as yet unclassified' stamp: 'zz 2/5/2009 10:50'!
type
	^ type! !

!PlayerMsgHdr methodsFor: 'as yet unclassified' stamp: 'zz 2/5/2009 14:15'!
type: aTypeNumber 
	type := aTypeNumber! !

Object subclass: #PlayerPose2d
	instanceVariableNames: 'px py pa'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Interfaces'!

!PlayerPose2d commentStamp: 'SergeStinckwich 3/24/2009 14:44' prior: 0!
/** @brief A pose in the plane */
typedef struct player_pose2d
{
  /** X [m] */
  double px;
  /** Y [m] */
  double py;
  /** yaw [rad] */
  double pa;
} player_pose2d_t;
!

!PlayerPose2d methodsFor: 'accessing' stamp: 'zz 2/6/2009 11:25'!
pa
	^ pa! !

!PlayerPose2d methodsFor: 'accessing' stamp: 'zz 2/6/2009 11:25'!
pa: aNumber 
	pa := aNumber! !

!PlayerPose2d methodsFor: 'accessing' stamp: 'zz 2/6/2009 11:25'!
px
	^ px! !

!PlayerPose2d methodsFor: 'accessing' stamp: 'zz 2/6/2009 11:25'!
px: aNumber 
	px := aNumber! !

!PlayerPose2d methodsFor: 'accessing' stamp: 'zz 2/6/2009 11:25'!
py
	^ py! !

!PlayerPose2d methodsFor: 'accessing' stamp: 'zz 2/6/2009 11:25'!
py: aNumber 
	py := aNumber! !

Object subclass: #PlayerPose3d
	instanceVariableNames: 'px py pz proll ppitch pyaw'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Interfaces'!

!PlayerPose3d commentStamp: 'SergeStinckwich 3/24/2009 14:47' prior: 0!
/** @brief A pose in space */
typedef struct player_pose3d
{
  /** X [m] */
  double px;
  /** Y [m] */
  double py;
  /** Z [m] */
  double pz;
  /** roll [rad] */
  double proll;
  /** pitch [rad] */
  double ppitch;
  /** yaw [rad] */
  double pyaw;
} player_pose3d_t;
!

Object subclass: #PlayerPosition2dData
	instanceVariableNames: 'vel pos stall'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Interfaces'!

!PlayerPosition2dData commentStamp: 'SergeStinckwich 3/24/2009 14:13' prior: 0!
/** position2d data */
typedef struct player_position2d_data
{
  /** position [m,m,rad] (x, y, yaw)*/
  player_pose2d_t pos;
  /** translational velocities [m/s,m/s,rad/s] (x, y, yaw)*/
  player_pose2d_t vel;
  /** Are the motors stalled? */
  uint8_t stall;
} player_position2d_data_t;
!

!PlayerPosition2dData methodsFor: 'accessing' stamp: 'SergeStinckwich 2/19/2009 15:42'!
pos
	^ pos! !

!PlayerPosition2dData methodsFor: 'accessing' stamp: 'SergeStinckwich 2/19/2009 15:48'!
pos: aPosition 
	pos := aPosition! !

!PlayerPosition2dData methodsFor: 'accessing' stamp: 'SergeStinckwich 2/24/2009 16:31'!
stall
	^ stall! !

!PlayerPosition2dData methodsFor: 'accessing' stamp: 'SergeStinckwich 2/24/2009 16:31'!
stall: b 
	stall := b! !

!PlayerPosition2dData methodsFor: 'accessing' stamp: 'SergeStinckwich 2/19/2009 15:42'!
vel
	^ vel! !

!PlayerPosition2dData methodsFor: 'accessing' stamp: 'SergeStinckwich 2/19/2009 15:47'!
vel: aVelocity
 vel := aVelocity! !

Object subclass: #PlayerPosition2dGeom
	instanceVariableNames: 'pose boundingBox2d'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Interfaces'!

!PlayerPosition2dGeom commentStamp: 'SergeStinckwich 3/24/2009 14:16' prior: 0!
/** position2d geom */
typedef struct player_position2d_geom
{
  /** Pose of the robot base, in the robot cs (m, rad). */
  player_pose3d_t pose;
  /** Dimensions of the base (m). */
  player_bbox3d_t size;
} player_position2d_geom_t;
!

!PlayerPosition2dGeom methodsFor: 'accessing' stamp: 'SergeStinckwich 3/24/2009 15:14'!
boundingBox2d
	^ boundingBox2d! !

!PlayerPosition2dGeom methodsFor: 'accessing' stamp: 'SergeStinckwich 3/24/2009 15:14'!
boundingBox2d: aPlayerBoundingBox2d 
	boundingBox2d := aPlayerBoundingBox2d! !

!PlayerPosition2dGeom methodsFor: 'accessing' stamp: 'SergeStinckwich 3/24/2009 15:11'!
pose
	^ pose! !

!PlayerPosition2dGeom methodsFor: 'accessing' stamp: 'SergeStinckwich 3/24/2009 15:11'!
pose: aPlayerPose 
	pose := aPlayerPose! !

Object subclass: #PlayerSimulationData
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Interfaces'!

Object subclass: #PlayerSonarData
	instanceVariableNames: 'rangesCount ranges'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Interfaces'!

!PlayerSonarData methodsFor: 'accessing' stamp: 'SergeStinckwich 2/27/2009 15:47'!
ranges
	^ ranges! !

!PlayerSonarData methodsFor: 'accessing' stamp: 'SergeStinckwich 2/27/2009 15:47'!
ranges: anArray 
	ranges := anArray! !

!PlayerSonarData methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 2/27/2009 15:46'!
rangesCount
	^ ranges size! !

Object subclass: #PlayerWifiData
	instanceVariableNames: 'throughput bitrate mode qualType maxqual maxlevel maxnoise ap links'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Interfaces'!

!PlayerWifiData methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 14:10'!
ap: aString 
	ap := aString! !

!PlayerWifiData methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 14:05'!
bitrate: anInteger

bitrate := anInteger! !

!PlayerWifiData methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/2/2009 14:24'!
initialize
	ap := String new: 32! !

!PlayerWifiData methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 10:11'!
links: aSetOfWifiLinks 
	links := aSetOfWifiLinks! !

!PlayerWifiData methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 14:06'!
maxlevel: anInteger 
	maxlevel := anInteger! !

!PlayerWifiData methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 14:07'!
maxnoise: anInteger 
	maxnoise := anInteger! !

!PlayerWifiData methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 14:06'!
maxqual: anInteger 
	maxqual := anInteger! !

!PlayerWifiData methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 14:05'!
mode: anInteger 
	mode := anInteger! !

!PlayerWifiData methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 14:06'!
qualType: anInteger 
	qualType := anInteger! !

!PlayerWifiData methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 14:04'!
throughput: anInteger
	throughput := anInteger! !

Object subclass: #PlayerWifiLink
	instanceVariableNames: 'mac ip essid mode freq encrypt qual level noise'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PlayerST-Interfaces'!

!PlayerWifiLink methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 10:01'!
essid: aByteArray 
	essid := aByteArray! !

!PlayerWifiLink methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 10:01'!
ip: aByteArray 
	ip := aByteArray! !

!PlayerWifiLink methodsFor: 'as yet unclassified' stamp: 'SergeStinckwich 3/3/2009 10:01'!
mac: aByteArray 
	mac := aByteArray! !

!String methodsFor: '*PlayerST' stamp: 'zz 3/19/2009 10:53'!
xdrString
	^ self size xdrUnsignedInteger , self asByteArray
		, (ByteArray new:  4 - (self size \\ 4))! !
PlayerClient initialize!
